/**
 *
 * Programmbeschreibung:
 * ---------------------
 * Dieses Programm ermittelt ber eine TCP/IP-Verbindung die Uhrzeit eines
 * entfernten Rechners, wobei es sowohl die Rolle des Servers als auch 
 * des Clients einnehmen kann.
 * Auf dem Rechner, von dem die Uhrzeit ermittelt werden soll, mu dieses 
 * Programm im Modus Server (Parameter -s) gestartet werden. Dann wartet 
 * es auf dem als Parameter bergebenen Port auf Anforderungen anderer 
 * Rechner und sendet das aktuelle Datum und Uhrzeit in Form einer 
 * Zeichenkette zurck.
 * Im Modus Client (Parameter -c) baut es eine Verbindung zum Port des 
 * bergebenen Rechners auf und gibt die zurckgemeldete Uhrzeit aus.
 *
 * @author  Peter Ren Dietmller
 * @version 1.0, 01/99
 * 
 */

import java.io.*;
import java.net.*;
import java.text.DateFormat;
import java.util.Date;

public class Time {

  /* -- Time Server -- */
  public static void StartServer(int port) {
  
    Socket       client = null;  /* Socket des Clients      */
    String       cip;            /* IP-Adresse des Clients  */
    int          cport;          /* Port des Clients        */
    PrintWriter  cout;           /* Ausgabestrom zum Client */
    ServerSocket server = null;  /* Socket des Servers      */
    String       time;           /* versendete Uhrzeit      */
    
    /* -- Server starten -- */
    System.out.println("Starte Server und warte auf Port " + port + " ... (Mit Strg-C beenden)");
    try {

      server = new ServerSocket(port);
      while(true) {

        /* -- Auf eingehende Verbindung warten -- */
        client = server.accept();
        cip   = client.getInetAddress().getHostAddress();
        cport = client.getPort();
        cout  = new PrintWriter(client.getOutputStream(), true);

        /* -- Uhrzeit ermitteln -- */
        time = DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.FULL).format(new Date());

        /* -- Uhrzeit senden -- */
        cout.println(time);
        
        /* -- Trace ausgeben -- */
        System.out.println(cip + ":" + cport + " " + time);
        
        /* -- Verbindung schlieen -- */
        client.close();
        
      }
      
    } catch(IOException e) {
      System.out.println("Server: " + e.toString());
      
    } finally {
      /* -- Socket schlieen -- */
      try {
        if (server != null) server.close();
        System.out.println("Server gestoppt.");
      } catch (IOException e) {
        System.out.println("Server.close(): " + e.toString());
      }
    }
    
  }
  
  /* -- Time Client: Abfrage der Uhrzeit -- */
  public static void StartClient(String server, int port) {
  
    String         line;
    BufferedReader in;
    Socket         s;
    
    System.out.println("Baue Verbindung zum Rechner " + server + ":" + port + " auf.");
    
    try {
    
      /* -- Verbindungsaufbau -- */
      s = new Socket(server, port);
      s.setSoTimeout(1000);
      in = new BufferedReader(new InputStreamReader(s.getInputStream()));
      System.out.println("Verbindung zum Rechner aufgebaut.");
      
      /* -- Lese Uhrzeit des Servers -- */
      line = in.readLine();
      if (line != null) {
        System.out.println("Die Uhrzeit ist: " + line);
      }
      
      /* -- Schliee Verbindung -- */
      s.close();
      
    } catch(UnknownHostException e) {
      System.out.println("Der Server " + server + " ist unbekannt.");

    } catch(IOException e) {
      System.out.println("Timeout");
    }

  }
  
  /* -- Hilfetext -- */
  public static void Usage() {
    System.out.println();
    System.out.println("Time ermittelt die Uhrzeit eines anderen Rechners.");
    System.out.println("(C)opryright by Peter Ren Dietmller, 1998-99.");
    System.out.println();
    System.out.println("Damit die Uhrzeit eines entfernten Rechners abgefragt");
    System.out.println("werden kann, mu dieses Programm auf dem entfernten Rechner");
    System.out.println("in der Betriebsart Server laufen.");
    System.out.println();
    System.out.println("Aufruf: java Time -s <port> | -c <server> <port>");
    System.out.println();
    System.out.println("  -s ........ Programm wird in der Betriebsart Server");
    System.out.println("              gestartet. In diesem Modus wartet das");
    System.out.println("              Programm auf Anfragen und sendet die Uhrzeit");
    System.out.println("              des Rechners, auf dem es luft.");
    System.out.println("  -c ........ Programm wird in der Betriebsart Client");
    System.out.println("              gestartet. In diesem Modus wird die Uhrzeit");
    System.out.println("              eines anderen Rechners abgefragt.");
    System.out.println("  <server> .. Name oder IP-Adresse des Servers.");
    System.out.println("  <port> .... Portnummer, auf der die Kommunikation");
    System.out.println("              stattfinden soll.");
    System.out.println();
  }


  /* -- Hauptprogramm -- */
  public static void main(String[] args) {
  
    int         port;
    String      server;
  
    /* -- Parameter prfen -- */
    if (args.length >= 1) {
    
      /* -- Server -- */
      if (args[0].equalsIgnoreCase("-s")) {
        if (args.length >= 2) {
          try {
            port = Integer.parseInt(args[1]);
            StartServer(port);
          } catch(NumberFormatException e) {
            System.out.println("Sie haben als Port keine Zahl angegeben.");
            Usage();
          }
        } else {
          System.out.println("Sie haben zu wenig Parameter angegeben.");
          Usage();
        }

      /* -- Client -- */
      } else if (args[0].equalsIgnoreCase("-c")) {
        if (args.length >= 3) {
          server = args[1];
          try {
            port = Integer.parseInt(args[2]);
            StartClient(server, port);
          } catch(NumberFormatException e) {
            System.out.println("Sie haben als Port keine Zahl angegeben.");
            Usage();
          }
        } else {
          System.out.println("Sie haben zu wenig Parameter angegeben.");
          Usage();
        }

      /* -- Falscher erster Parameter -- */
      } else {
        System.out.println("Der erste Parameter ist falsch.");
        Usage();
      }
      
    /* -- args.length < 1 --> Falsche Anzahl der Parameter -- */
    } else {
      System.out.println("Sie haben zu wenig Parameter angegeben.");
      Usage();
    }
  }

}
